#include <algorithm>
#include <cstdio>
#include <cstdlib>
#include <list>
#include <set>
#include "findnumber.h"

// Judge secret generation. Change constants before reuse!
//
static long long judge_generateSecret(long long p) {
    const long long base = 137831;
    // prime modulo
    //
    const long long modulo = 1000000207;
    long long res = 1;
    long long v = base;
    while (p) {
        if (p & 1) {
            res = (res * v) % modulo;
        }
        v = (v * v) % modulo;
        p /= 2;
    }
    return res;
}

// Interval is [0..N)
// 0 <= l, r <= N
// [l, r) iff l < r
// [l, N) U [0, r) iff l > r
// Contestant function.
//
//long long play(long long N);

struct cat_query {
    long long l, r, ans;
};

static std::vector<cat_query> cat_queries;

// Store only simple intervals.
//
struct cat_interval {
    long long l, r;
};

static int cat_steps = 0;
static long long cat_N;
static int cat_maxsteps;
// Single interval assuming everything is true.
//
static std::list<cat_interval> cat_A;
// The set of intervals where the answer might be. Disjoint.
//
static std::list<cat_interval> cat_B;

static cat_interval cat_inters_simple(cat_interval a, long long l, long long r) {
    cat_interval res;
    res.l = std::max(a.l, l);
    res.r = std::min(a.r, r);
    if (res.l >= res.r) {
        res.l = res.r = -1;
    }
    return res;
}

// Remove [l, r) from a - simple intervals.
//
static std::list<cat_interval> cat_sub_simple(cat_interval a, long long l, long long r) {
    std::list<cat_interval> res;
    if (a.l < l) {
        res.push_back({a.l, std::min(l, a.r)});
    }
    if (r < a.r) {
        res.push_back({std::max(a.l, r), a.r});
    }
    return res;
}

static void cat_intersect(const std::list<cat_interval> &s, long long l, long long r,
        std::list<cat_interval> &common, std::list<cat_interval> &disjoint) {
    if (l == r) {
        disjoint = s;
        return;
    }

    for (std::list<cat_interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        if (l < r) {
            cat_interval com = cat_inters_simple(*it, l, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            disjoint.splice(disjoint.begin(), cat_sub_simple(*it, l, r));
        } else if (l > r) {
            // Spawn at most one extra among all here.
            //
            std::list<cat_interval> dis;
            cat_interval com = cat_inters_simple(*it, l, cat_N);
            if (com.l != -1) {
                common.push_back(com);
            }
            dis = cat_sub_simple(*it, l, cat_N);
            com = cat_inters_simple(*it, 0, r);
            if (com.l != -1) {
                common.push_back(com);
            }
            for (std::list<cat_interval>::const_iterator jt = dis.begin();
                    jt != dis.end(); ++jt) {
                disjoint.splice(disjoint.begin(), cat_sub_simple(*jt, 0, r));
            }
        }
    }
}

static long long cat_set_size(const std::list<cat_interval> &s) {
    long long sz = 0;
    for (std::list<cat_interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        sz += it->r - it->l;
    }
    return sz;
}

static long long cat_eval(long long a, long long b) {
    if (a == 1 && b == 0) {
        return 0;
    }
    if (a == 0 && b == 1) {
        return 0;
    }
    if (!a && !b) {
        fprintf(stderr, "Invalid state!\n");
        return -1;
    }
    for (long long k = 1; k <= 60; ++k) {
        long long e = 0;
        long long tpow = (1LL << (k - 1));
        if ((tpow + 1) % (k + 1) == 0) {
            e = 1;
        }
        long long mk = 2 * (tpow / (k + 1)) + e;
        // printf("A? %lld %lld %lld -> %lld\n", a, b, k, mk);
        if (a > mk) {
            continue;
        }
        if (a * (k + 1) + b <= (1LL << k)) {
            // printf("EVAL %lld %lld -> %lld\n", a, b, k);
            return k;
        }
    }
    return 62;
}

void pstate(const std::list<cat_interval> &s) {
    for (std::list<cat_interval>::const_iterator it = s.begin(); it != s.end(); ++it) {
        printf("[%lld, %lld) ", it->l, it->r);
    }
    printf("\n");
}

// Method available to contestant
// Ignoring x and flip.
//
bool isInside(long long l, long long r) {
    ++cat_steps;
    std::list<cat_interval> A1, A2, B1, B2;
    cat_intersect(cat_A, l, r, A1, A2);
    cat_intersect(cat_B, l, r, B1, B2);

    long long sly, sln;
    sly = cat_eval(cat_set_size(A1), cat_set_size(A2) + cat_set_size(B2));
    sln = cat_eval(cat_set_size(A2), cat_set_size(A1) + cat_set_size(B1));

    if (sly > sln || sln == -1) {
        cat_A = A1;
        cat_B = A2;
        cat_B.splice(cat_B.begin(), B2);
        cat_queries.push_back({l, r, 1LL});
        // fprintf(stderr, "ASK %lld %lld -> %lld\n", l, r, 1LL);
        return 1;
    } else {
        cat_A = A2;
        cat_B = A1;
        cat_B.splice(cat_B.begin(), B1);
        cat_queries.push_back({l, r, 0LL});
        // fprintf(stderr, "ASK %lld %lld -> %lld\n", l, r, 0LL);
        return 0;
    }
}

static void cat_printQueries() {
    fprintf(stderr, "Queries id: l r ans\n");
    for (int i = 0; i < (int) cat_queries.size(); ++i) {
        fprintf(stderr, "%d: %lld %lld %lld\n",
                i, cat_queries[i].l, cat_queries[i].r, cat_queries[i].ans);
    }
}

static bool cat_isLie(cat_query q, long long ans) {
    if (q.l == q.r) {
        return q.ans;
    } else if (q.l < q.r) {
        if (q.l <= ans && ans < q.r) {
            return 1 ^ q.ans;
        } else {
            return q.ans;
        }
    } else {
        if (q.l <= ans || ans < q.r) {
            return 1 ^ q.ans;
        } else {
            return q.ans;
        }
    }
}

static void done(long long x) {
    long long ans = 0;
    if (cat_eval(cat_set_size(cat_A), cat_set_size(cat_B)) == 0) {
        if (cat_A.size()) {
            ans = cat_A.begin()->l;
        } else if (cat_B.size()) {
            ans = cat_B.begin()->l;
        }
    } else {
        std::set<long long> s;
        for (auto a : cat_A) {
            for (long long x = a.l; x != a.r && (int) s.size() < 2;
                    x = (x + 1) % cat_N) {
                s.insert(x);
            }
        }
        for (auto a : cat_B) {
            for (long long x = a.l; x != a.r && (int) s.size() < 2;
                    x = (x + 1) % cat_N) {
                s.insert(x);
            }
        }
        if (!s.size()) {
            cat_printQueries();
            fprintf(stderr, "Checker is incorrect! No possible answer!\n");
            return;
        }
        ans = *s.begin();
        if (ans == x) {
            ans = *s.rbegin();
        }
    }
    // Check if ans satisfies constraints!
    //

    int isLie = -1;
    for (int i = 0; i < (int) cat_queries.size(); ++i) {
        if (cat_isLie(cat_queries[i], ans)) {
            if (isLie == -1) {
                isLie = i;
            }
        } else {
            if (isLie != -1) {
                cat_printQueries();
                fprintf(stderr, "Checker is incorrect! Told truth on move %d when started "
                        "lying on move %d\n", i, isLie);
                return;
            }
        }
    }

    if (x != ans) {
        printf("-1 %lld %lld\n",x,ans);
        //fprintf(stderr, "Wrong Answer guessed: %lld, correct: %lld\n", x, ans);
    } else if (cat_steps > cat_maxsteps) {
        printf("-2 %d %d\n", cat_steps, cat_maxsteps);
        //fprintf(stderr, "Too many steps. Used: %d Available: %d\n",
        //        cat_steps, cat_maxsteps);
    } else {
        printf("%lld\n", judge_generateSecret(cat_N + cat_maxsteps));
    }
    exit(0);
}

int main() {
    if (scanf("%lld %d", &cat_N, &cat_maxsteps) != 2) {
        fprintf(stderr, "Invalid input!\n");
        return 0;
    }
    cat_interval A;
    A.l = 0;
    A.r = cat_N;
    cat_A.push_back(A);

    done(play(cat_N));
    printf("No answer received\n");
    return 0;
}
